#!/usr/bin/env python3
# coding=utf-8

'''
Name: tui media player
Date: 2024/10/27
Description:
Author: XQTR
Website: https://cp737.net
'''

import os,sys
from mpycrt import *
import struct
import time
import subprocess
import threading
import datetime
import re
import json

COLS=os.get_terminal_size().columns
LINES=os.get_terminal_size().lines

QUIT=False
PLAY=False
SHOWSPIN=True
NOKEYS=False
BARWIDTH=10
SHUFFLE=False
CONFIG={'shuffle':False,'repeat':False,'next':True}
OFFSET=(int(COLS)-60) // 2

spin=['|','/','-','\\']
spinindex=0
diskindex=0

def show_file(fname):
  global SHOWSPIN,OFFSET
  clrscr()
  backspin = SHOWSPIN
  SHOWSPIN = False
  with open(fname) as fn:
    for l in fn:
      write(' '*OFFSET+l)
  readkey()
  draw()
  SHOWSPIN = backspin

def getcapslock():
  s = os.popen('xset -q').read()
  if re.search('Caps Lock:\s+on', s):
    return 'on'
  else:
    return 'off'
    
def savelist2json(lista,filename):
  try:
    with open(filename, 'w+') as outfile:
      json.dump(lista, outfile,indent=2)
    return 0
  except:
    return -1
    
def loadjson2list(filename):
  try:
    if os.path.exists(filename):
      with open(filename) as json_file:
        jlist = json.load(json_file)
        return jlist
    else:
      return None
  except:
    pass
    
def draw_disk_spin():
  global spin,spinindex,diskindex,PLAY,SHOWSPIN,OFFSET
  if SHOWSPIN==False:return
  #if not PLAY: return
  spinindex+=1
  if spinindex>=len(spin):
    spinindex = 0
  writexy(19+OFFSET,9,8,spin[spinindex])
  writexy(41+OFFSET,9,8,spin[spinindex])
  
  diskindex+=1
  if diskindex>=29:
    diskindex=1
    writexy(44,12,8,'#')
  writexy(16+OFFSET+diskindex-1,12,8,'#')
  writexy(16+OFFSET+diskindex,12,7,'#')
  
  gotoxy(1,1)
  time.sleep(1)
  
def formatsec(n):
  m, s = divmod(n, 60)
  h, m = divmod(m, 60)
  if h == 0:
    return str(f'{m:02d}:{s:02d}')
  else:
    return str(f'{h:d}:{m:02d}:{s:02d}')


    
def thread_redraw():
  while True:
    if PLAY:
      draw_disk_spin()
    draw_song()


def center(y, tw, a, s):
  #w = len(s) // 2
  w = tw // 2
  writexy(COLS // 2 - w, y, a, s)
  
def draw():
  global OFFSET
  clrscr()
  writeln(" "*OFFSET+"[0m")
  writeln(" "*OFFSET+"        [1m_______[36m_____[37m_[36m_________[37m_[36m____________________")
  writeln(" "*OFFSET+"[0m       [1m|[0m  [1m_____[0m__________________________________  [1;36m|[0m")
  writeln(" "*OFFSET+"      [1;30m:[37m|[0m [1m/[0m [1;30m.-----------------------------------.[0m \ [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[37m|[0m [1m|[0m [1;30m| /\[0m [1;30m:[0m                        [1;30m90 min|[0m [1;30m|[0m [36m|:[37m")
  writeln(" "*OFFSET+"      [1;30m:[37m|[0m [1m|[0m [1;30m|/--\:....................... NR [ ]|[0m [1;30m|[0m [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m [1m|[0m [1;30m`-----------------------------------'[0m [1;30m|[0m [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m [1m|[0m      [1;30m//-\\\\[0m   [1;30m|         |[0m   [1;30m//-\\\\[0m      [1;30m|[0m [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m |     [1;30m||[0m | [1;30m||[0m  [1;30m|_________|[0m  [1;30m||[0m | [1;30m||[0m     | [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m |      [1;30m\\\\-//[0m   [1;30m:....:....:[0m   [1;30m\\\\-//      |[0m [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[37m|[0m |                                       [1;30m|[0m [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m |     [1;30m#############################[0m     [1;30m|[0m [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m |                                       [1;30m|[0m [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m `______ [1;30m_________[0m_[1;30m__________ _[0m_[1;30m__ ______'[0m [36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m        [1;30m/    []             []    \        [0;36m|[1;30m:[0m")
  writeln(" "*OFFSET+"      [1;30m:[36m|[0m       [1;30m/[0m  [1;30m()                   ()  \       [0;36m|[1;30m:[0m")
  writeln(" "*OFFSET+"       [1;36m!_____[0m_[1;30m/[0;36m_____________________________[1;30m\[0;36m______![37m")
  writeln("")
  writeln(" "*OFFSET+"                    [36m_____________________")
  writeln(" "*OFFSET+"[37m                  [36m/`[37m    [36m00:00 | 00:00[37m    [36m`\[37m")
  writeln(" "*OFFSET+"  [36m.--------------+------------------------+--------------.")
  writeln(" "*OFFSET+"[37m [36m:|[37m                                                      [36m|:[37m")
  writeln(" "*OFFSET+" [36m:|[37m [36m====================================================[37m [36m|:[37m")
  writeln(" "*OFFSET+"  [36m`--------------+------------------------+--------------'")
  writeln(" "*OFFSET+"[37m     [36m\[37m  [36mxxx[37m [36mKbps[37m [36m|[37m                        [36m|[37m    [36mxx[37m [36mkHz[37m [36m/[37m")
  writeln(" "*OFFSET+"      [36m\----------+------------------------+----------/[37m")
  writeln("")
  writeln("")
  write(" "*OFFSET+"                      [1mH[30melp [0;36m|[1;30m [37mK[30meys [0;36m|[1;30m [37mQ[30muit[0m")
  

 # %state     State
 # %file      File
 # %title     Title
 # %artist    Artist
 # %song      SongTitle
 # %album     Album
 # %tt        TotalTime
 # %tl        TimeLeft
 # %ts        TotalSec
 # %ct        CurrentTime
 # %cs        CurrentSec
 # %b         Bitrate
 # %r         Rate

  
def draw_song():
  global PLAY,BARWIDTH,SHOWSPIN,OFFSET
  if SHOWSPIN==False: return
  try:
    result = subprocess.run(['mocp', '-Q %state;%title;%artist;%song;%album;%ts;%cs;%b;%r;%file'], stdout=subprocess.PIPE)
    res = result.stdout.decode().split(';')
    writexy(26+OFFSET,8,11,res[0].strip().center(9,' '))
    if "PLAY" in res[0]:
      PLAY = True
    else:
      PLAY = False
    title = res[1]
    artist = res[2]
    song = res[3]
    album = res[4]
    ts = int(res[5])
    cs = int(res[6])
    bitrate = res[7]
    rate = res[8]
    fname = res[9]
    
    banner = ""  
    if title=="":
      if song == "":
        banner = fname.split('/')[-1] 
      else:
        banner = song + ' [' + album +']'
    else:
      banner = title
    center(22,52,15,banner[:COLS-11].center(COLS-11, ' '))
    
    
    scs=formatsec(cs)
    sts=formatsec(ts)
    BARWIDTH = 52
    #writexy(25,20,8,colors[15]+scs+colors[3]+' | '+colors[3]+sts)
    #center(20,21,8,' '*21)
    #center(20,21,8,colors[15]+scs+colors[3]+' | '+colors[3]+sts)
    
    writexy(22+OFFSET,20,15,scs.center(8,' '))
    writexy(33+OFFSET,20,3,sts.center(8,' '))
    
    x = cs * BARWIDTH // ts
    writexy(5+OFFSET,23,11,str('#'*x)+colors[8]+str('='*(BARWIDTH-x)))
    
    writexy(9+OFFSET,25,10,colors[15]+bitrate)
    writexy(48+OFFSET,25,10,colors[15]+rate)
    
    
        
  except:
    pass

def saveconfig():
  global CONFIG
  savelist2json(CONFIG,"config.cfg")
  
def loadconfig():
  global CONFIG
  CONFIG = loadjson2list('config.cfg')
  if CONFIG['shuffle']:
    os.system('mocp -o shuffle')
  else:
    os.system('mocp -u shuffle')
  if CONFIG['repeat']:
    os.system('mocp -o repeat')
  else:
    os.system('mocp -u repeat')
  if CONFIG['next']:
    os.system('mocp -o autonext')
  else:
    os.system('mocp -u autonext')
  
def main():
  global SHOWSPIN,PLAY,QUIT,SHUFFLE,CONFIG
 
  os.system('mocp -S')
  loadconfig()
  draw()
  refresh_thread = threading.Thread(target = thread_redraw)
  refresh_thread.daemon=True
  refresh_thread.start()
  
  while True:
    control = ""
    
    if CONFIG['shuffle']: control+='SHUFFLE '
    if CONFIG['repeat']: control+='REPEAT '
    if CONFIG['next']: control+='AUTONEXT '
    writexy(19+OFFSET,25,11,' '*24)
    writexy(19+OFFSET,25,11,control.center(24,' '))
    
    ch = readkey()

    if ch in "qQ": 
      QUIT=True
    elif ch in "mM":
      backspin = SHOWSPIN
      SHOWSPIN = False
      os.system('mocp -T rteff_theme')            
      draw()
      SHOWSPIN = backspin
      
    
    elif ch =="1":
      os.system("amixer set Master 10% >/dev/null 2>&1")
    elif ch =="2":
      os.system("amixer set Master 20% >/dev/null 2>&1")
    elif ch =="3":
      os.system("amixer set Master 30% >/dev/null 2>&1")
    elif ch =="4":
      os.system("amixer set Master 40% >/dev/null 2>&1")
    elif ch =="5":
      os.system("amixer set Master 50% >/dev/null 2>&1")
    elif ch =="6":
      os.system("amixer set Master 60% >/dev/null 2>&1")
    elif ch =="7":
      os.system("amixer set Master 70% >/dev/null 2>&1")
    elif ch =="8":
      os.system("amixer set Master 80% >/dev/null 2>&1")
    elif ch =="9":
      os.system("amixer set Master 90% >/dev/null 2>&1")
    elif ch =="0":
      os.system("amixer set Master 100% >/dev/null 2>&1")
    elif ch == "#space": #toggle play/pause
      PLAY = not PLAY 
      os.system('mocp -G')
    elif ch == "#left": 
      os.system('mocp --seek=-5')
    elif ch == "#right": 
      os.system('mocp --seek=5')
    elif ch == "#down": 
      os.system('mocp --seek=-20')
    elif ch == "#up": 
      os.system('mocp --seek=20')
    elif ch in "pP": #play
      os.system('mocp --play')
    elif ch in "sS": #stop
      os.system('mocp -s')
    elif ch in "cC": #clear playlist
      os.system('mocp -c')
    elif ch in "aA": #Alsamixer
      backspin = SHOWSPIN
      SHOWSPIN = False
      os.system('alsamixer')
      draw()
      SHOWSPIN = backspin
      
    elif ch in "nN": #next
      os.system('mocp -f')
    elif ch in "bB": #previous
      os.system('mocp -r')
    elif ch in "cC": #clear
      os.system('mocp -c')
    elif ch in "fF": #shuffle
      CONFIG['shuffle'] = not CONFIG['shuffle']
      if CONFIG['shuffle']:
        os.system('mocp -o shuffle')
      else:
        os.system('mocp -u shuffle')
    elif ch in "Rr": #repeat
      CONFIG['repeat'] = not CONFIG['repeat']
      if CONFIG['repeat']:
        os.system('mocp -o repeat')
      else:
        os.system('mocp -u repeat')
    elif ch in "Xx": #autonext
      CONFIG['next'] = not CONFIG['next']
      if CONFIG['next']:
        os.system('mocp -o autonext')
      else:
        os.system('mocp -u autonext')
    elif ch in "kK":
      show_file("keys.ans")
    elif ch in "hH":
      show_file("help.ans")
    elif ch == "-":
      os.system("amixer set Master 5%- >/dev/null 2>&1")
    elif ch == "+":
      os.system("amixer set Master 5%+ >/dev/null 2>&1")
    
    
    

    if QUIT:
      termios.tcflush(sys.stdin, termios.TCIOFLUSH)
      PLAY=False
      saveconfig()
      os.system('mocp -x')
      clrscr()
      exit()
    
  

if __name__ == "__main__":
  main()
  PLAY=False
  

