#!/usr/bin/python3

import os, sys, time
import tty,termios

colors = {
'r':"reset='\033[0m",
  0:"\033[0;30m",
  1:"\033[0;34m",
  2:"\033[0;32m",
  3:"\033[0;36m",
  4:"\033[0;31m",
  5:"\033[0;35m",
  6:"\033[0;33m",
  7:"\033[0;37m",
  8:"\033[1;30m",
  9:"\033[1;34m",
  10:"\033[1;32m",
  11:"\033[1;36m",
  12:"\033[1;31m",
  13:"\033[1;35m",       
  14:"\033[1;33m",
  15:"\033[1;37m",
  16:"\033[40m",
  17:"\033[44m",
  18:"\033[42m",
  19:"\033[46m",
  20:"\033[41m",
  21:"\033[45m",
  22:"\033[43m",      
  23:"\033[47m"
}

asciibox=('.','-','.','|','|','`','-','\'',' ')

# key codes used for the readkey() function below

# common
LF = '\x0d'
CR = '\x0a'
ENTER = '\x0d'
BACKSPACE = '\x7f'
SUPR = ''
SPACE = '\x20'
ESC = '\x1b'

# other
F1 = '\x1b\x4f\x50'
F2 = '\x1b\x4f\x51'
F3 = '\x1b\x4f\x52'
F4 = '\x1b\x4f\x53'
F5 = '\x1b\x4f\x31\x35\x7e'
F6 = '\x1b\x4f\x31\x37\x7e'
F7 = '\x1b\x4f\x31\x38\x7e'
F8 = '\x1b\x4f\x31\x39\x7e'
F9 = '\x1b\x4f\x32\x30\x7e'
F10 = '\x1b\x4f\x32\x31\x7e'
F11 = '\x1b\x4f\x32\x33\x7e'
F12 = '\x1b\x4f\x32\x34\x7e'

PAGE_UP = '\x1b\x5b\x35\x7e'
PAGE_DOWN = '\x1b\x5b\x36\x7e'
HOME = '\x1b\x5b\x48'
END = '\x1b\x5b\x46'

INSERT = '\x1b\x5b\x32\x7e'
SUPR = '\x1b\x5b\x33\x7e'

pathchar = os.sep
pathsep  = os.sep

fdInput = sys.stdin.fileno()
termAttr = termios.tcgetattr(0) 

wherex = 1
wherey = 1
attr = 7
oldx   = 1
oldy   = 1
utf8  = True

rows, columns = os.popen('stty size', 'r').read().split()
height = int(rows)
width = int(columns)

def delay(t):
  time.sleep(t/ 1000.0)
  
def byte2str(v):
    s=''.join(str(v))
    return s[2:-1]

def textcolor(c):
  global attr
  bg = c // 16
  fg = c % 16
  attr = c
  sys.stdout.write(colors[fg]+colors[bg+16])
  sys.stdout.flush()
      
      
def gotoxy(x,y):
  global wherex,wherey
  if x < 1:
      x = 1
  if x > width:
      x = width
  if y<1:
      y = 1
  if y>height:
      y=height
  wherex = x
  wherey = y
  sys.stdout.write('\033['+str(y)+';'+str(x)+'H')
  sys.stdout.flush()
      
def clrscr():
  sys.stdout.write('\033[2J')
  sys.stdout.flush()
  gotoxy(1,1)
       
def savecursor():
  global oldx,oldy,wherex,wherey
  oldx = wherex
  oldy = wherey
  
def restorecursor():
  global oldx,oldy,wherex,wherey
  wherey = oldy
  wherex = oldx
    
   
def write(s):
  if utf8==True:
    sys.stdout.write(str(s))
    sys.stdout.flush()
  else:
    os.write(1,bytes(s,"CP437"))
  
def writeln(s):
  global wherex,wherey,utf8
  if utf8:
    sys.stdout.write(str(s)+'\n')
    sys.stdout.flush()
  else:
    os.write(1,bytes(s+'\n',"CP437"))
  wherex = 1
  wherey += 1
    
def writexy(x,y,a,s):
  gotoxy(x,y)
  textcolor(a)
  write(s)

def cleararea(x1,y1,x2,y2,bg):
  for i in range(y2-y1):
      gotoxy(x1,y1+i)
      write(bg*(x2-x1))
        
def box(x1,y1,x2,y2,box):
  gotoxy(x1,y1)
  write(box[0]+box[1]*(x2-x1-1)+box[2])
  gotoxy(x1,y2)
  write(box[5]+box[6]*(x2-x1-1)+box[7])
  for i in range(y2-y1-1):
    gotoxy(x1,y1+1+i)
    write(box[3]+box[8]*(x2-x1-1)+box[4])
        
def readkey():
  tty.setraw(fdInput)
  ch = sys.stdin.buffer.raw.read(4).decode(sys.stdin.encoding)
  if len(ch) == 1:
    if ch == ENTER:
      ch = "#enter"
    elif ch == BACKSPACE:
      ch = "#back"
    elif ch == SPACE:
      ch = "#space"
    elif ch == ESC:
      ch = "#esc"
    elif ord(ch) < 32 or ord(ch) > 126:
      ch = ord(ch)

  elif ch == INSERT:
    ch = "#ins"
  elif ch == PAGE_DOWN:
    ch = "#pgdn"
  elif ch == PAGE_UP:
    ch = "#pgup"
  elif ch == HOME:
    ch = "#home"
  elif ch == END:
    ch = "#end"
  elif ch == F1:
    ch = "#f1"
  elif ch == F2:
    ch = "#f2"
  elif ch == F3:
    ch = "#f3"
  elif ch == F4:
    ch = "#f4"
  elif ch == F5:
    ch = "#f5"
  elif ch == F6:
    ch = "#f6"
  elif ch == F7:
    ch = "#f7"
  elif ch == F8:
    ch = "#f8"
  elif ch == F9:
    ch = "#f9"
  elif ch == F10:
    ch = "#f10"
  elif ch == F11:
    ch = "#f11"
  elif ch == F12:
    ch = "#f12"
  elif ord(ch[0]) == 27:
    if ch[1] == "[":
      if ch[2] == "A":
          ch = "#up"
      elif ch[2] == "B":
          ch = "#down"
      elif ch[2] == "C":
          ch = "#right"
      elif ch[2] == "D":
          ch = "#left"
      elif ch[2] == "K" or ch[2]=="F":
          ch = "#end"
      elif ch[2] == "V":
          ch = "#pgup"
      elif ch[2] == "U":
          ch = "#pgdn"
  else:
    return ch
  
  
  termios.tcsetattr(fdInput, termios.TCSADRAIN, termAttr)
  return ch        
    
    
